@@ -20,12 +20,15 @@ Raster and Vector I/O using GDAL
 from __future__ import absolute_import
 
 import os
+import deprecation
+from deprecation import deprecated
 
 # site packages
 from osgeo import gdal, ogr, osr
 
-# wradlib modules
-from .. import util as util
+from ..version import short_version
+
+deprecation.message_location = "top"
 
 
 def open_vector(filename, driver=None):
@@ -60,7 +63,9 @@ def open_vector(filename, driver=None):
     return dataset, layer
 
 
-@util.deprecated(open_vector)
+@deprecated(deprecated_in="0.11.1", removed_in="1.0.0",
+            current_version=short_version,
+            details="Use :func:`open_vector` instead.")
 def open_shape(filename, driver=None):
     """Open shapefile, return gdal.Dataset and OGR.Layer
 
