@@ -30,6 +30,8 @@ from time import mktime
 import warnings
 import functools
 import os
+import deprecation
+from deprecation import deprecated
 
 import numpy as np
 from scipy import interpolate
@@ -37,65 +39,14 @@ from scipy.ndimage import filters
 from scipy.spatial import cKDTree
 from osgeo import ogr
 
+from . version import short_version
+
+deprecation.message_location = "top"
+
 warnings.simplefilter('always', DeprecationWarning)
 # warnings.simplefilter('always', FutureWarning)
 
 
-def deprecated(replacement=None):
-    """A decorator which can be used to mark functions as deprecated.
-    replacement is a callable that will be called with the same args
-    as the decorated function.
-
-    Author: Giampaolo Rodola' <g.rodola [AT] gmail [DOT] com>
-    License: MIT
-
-    Parameters
-    ----------
-    replacement: string
-        function name of replacement function
-
-    >>> # output stderr on stdout, to capture warning message
-    >>> import sys
-    >>> save = sys.stderr
-    >>> sys.stderr = sys.stdout  # noqa
-
-    >>> from wradlib.util import deprecated
-    >>> @deprecated()
-    ... def foo(x):
-    ...     return x
-    >>> ret = foo(1) #doctest: +ELLIPSIS
-    /.../util.py:1: DeprecationWarning: wradlib.util.foo is deprecated
-      #!/usr/bin/env python
-
-    >>> def newfun(x):
-    ...     return 0
-    >>> @deprecated(newfun)
-    ... def foo(x):
-    ...     return x
-    >>> ret = foo(1) #doctest: +ELLIPSIS
-    /.../util.py:1: DeprecationWarning: wradlib.util.foo is deprecated; \
-use <function newfun at 0x...> instead
-      #!/usr/bin/env python
-
-    >>> # reset sys.stderr
-    >>> sys.stderr = save
-    """
-
-    def outer(fun):
-        msg = "%s.%s is deprecated" % (fun.__module__, fun.__name__)
-        if replacement is not None:
-            msg += "; use %s instead" % replacement
-
-        @functools.wraps(fun)
-        def inner(*args, **kwargs):
-            warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
-            return fun(*args, **kwargs)
-
-        return inner
-
-    return outer
-
-
 def apichange_kwarg(ver, par, typ, expar=None, exfunc=None, msg=None):
     """A decorator to generate a DeprectationWarning.
 
@@ -234,6 +185,8 @@ latest/gettingstarted.html#optional-dependencies
     return mod
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def aggregate_equidistant_tseries(tstart, tend, tdelta, tends_src, tdelta_src,
                                   src, method="sum", minpercvalid=100.):
     """Aggregates an equidistant time series to equidistant target time windows.
@@ -308,7 +261,7 @@ def aggregate_equidistant_tseries(tstart, tend, tdelta, tends_src, tdelta_src,
     >>> src = [1, 1, 1, 1, 1]
     >>> tstarts, tends, agg = aggregate_equidistant_tseries(tstart, tend, \
     tdelta, tends_src, tdelta_src, src, minpercvalid=50.)
-    >>> print(agg)
+    >>> print(agg) # doctest: +NORMALIZE_WHITESPACE
     [  4.  nan  nan  nan]
 
     """
@@ -383,6 +336,8 @@ def aggregate_equidistant_tseries(tstart, tend, tdelta, tends_src, tdelta_src,
     return tstarts, tends, agg
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def aggregate_in_time(src, dt_src, dt_trg, taxis=0, func='sum'):
     """Aggregate time series data to a coarser temporal resolution.
 
@@ -456,7 +411,8 @@ def aggregate_in_time(src, dt_src, dt_trg, taxis=0, func='sum'):
     2008-06-02 08:00:00
     2008-06-02 12:00:00
     >>> print('target time series') #doctest: +SKIP
-    >>> print(aggregate_in_time(src, dt_src, dt_trg, taxis=0, func='sum'))
+    >>> agg = aggregate_in_time(src, dt_src, dt_trg, taxis=0, func='sum')
+    >>> print(agg) # doctest: +NORMALIZE_WHITESPACE
     [[  24.   28.   32.   36.]
      [  88.   92.   96.  100.]
      [  nan   nan   nan   nan]]
@@ -483,6 +439,8 @@ def aggregate_in_time(src, dt_src, dt_trg, taxis=0, func='sum'):
     return trg
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def sum_over_time_windows(src, dt_src, dt_trg, minpercvalid):
     """Returns the sums of time series <src> within the time windows dt_trg
 
@@ -535,6 +493,8 @@ def sum_over_time_windows(src, dt_src, dt_trg, minpercvalid):
     return accum
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def mean_over_time_windows(src, dt_src, dt_trg, minbasepoints=1):
     """UNDER DEVELOPMENT: Aggregate time series data to a coarser temporal
     resolution.
@@ -604,6 +564,8 @@ def mean_over_time_windows(src, dt_src, dt_trg, minbasepoints=1):
     return trg
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def average_over_time_windows(src, dt_src, dt_trg, maxdist=3600,
                               helper_interval=300, **ipargs):
     """UNDER DEVELOPMENT: Computes the average of a time series over given
@@ -688,6 +650,8 @@ def average_over_time_windows(src, dt_src, dt_trg, maxdist=3600,
     return trg
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def _get_func(funcname):
     """
     Retrieve the numpy function with name <funcname>
@@ -747,6 +711,8 @@ def from_to(tstart, tend, tdelta):
     return tsteps
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def _tdelta2seconds(tdelta):
     """
     Convert a dt.timedelta object to seconds
@@ -759,6 +725,8 @@ def _tdelta2seconds(tdelta):
     return tdelta.days * 86400 + tdelta.seconds
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def _get_tdelta(tstart, tend, as_secs=False):
     """Returns the difference between two datetimes
     """
@@ -772,6 +740,8 @@ def _get_tdelta(tstart, tend, as_secs=False):
         return _tdelta2seconds(tend - tstart)
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def iso2datetime(iso):
     """Converts an ISO formatted time string to a datetime object.
 
@@ -796,6 +766,8 @@ def iso2datetime(iso):
         raise
 
 
+@deprecated(deprecated_in="0.11.3", removed_in="1.0.0",
+            current_version=short_version)
 def timestamp2index(ts, delta, refts, **kwargs):
     """Calculates the array index for a certain time in an equidistant
     time-series given the reference time (where the index would be 0)
@@ -1133,7 +1105,7 @@ def maximum_intensity_projection(data, r=None, az=None, angle=None,
 
 
 def filter_window_polar(img, wsize, fun, rscale, random=False):
-    r"""Apply a filter of an approximated square window of half size `fsize`
+    """Apply a filter of an approximated square window of half size `fsize` \
     on a given polar image `img`.
 
     Parameters
@@ -1202,7 +1174,7 @@ def prob_round(x, prec=0):
 
 
 def filter_window_cartesian(img, wsize, fun, scale, **kwargs):
-    r"""Apply a filter of square window size `fsize` on a given
+    """Apply a filter of square window size `fsize` on a given \
     cartesian image `img`.
 
     Parameters
@@ -1229,7 +1201,7 @@ def filter_window_cartesian(img, wsize, fun, scale, **kwargs):
 
 
 def roll2d_polar(img, shift=1, axis=0):
-    r"""Roll a 2D polar array [azimuth,range] by a given `shift` for
+    """Roll a 2D polar array [azimuth,range] by a given `shift` for \
     the given `axis`
 
     Parameters
@@ -1267,8 +1239,7 @@ def roll2d_polar(img, shift=1, axis=0):
 
 
 class UTC(tzinfo):
-    """
-    UTC implementation for tzinfo.
+    """UTC implementation for tzinfo.
 
     See e.g. http://python.active-venture.com/lib/datetime-tzinfo.html
 
@@ -1321,7 +1292,7 @@ def half_power_radius(r, bwhalf):
 
 
 def get_raster_origin(coords):
-    """
+    """Return raster origin
 
     Parameters
     ----------
@@ -1338,8 +1309,7 @@ def get_raster_origin(coords):
 
 
 def find_bbox_indices(coords, bbox):
-    """
-    Find min/max-indices for NxMx2 array coords using bbox-values.
+    """Find min/max-indices for NxMx2 array coords using bbox-values.
 
     The bounding box is defined by two points (llx,lly and urx,ury)
     It finds the first indices before llx,lly and the first indices
@@ -1439,7 +1409,7 @@ def calculate_polynomial(data, w):
 
     .. math::
 
-        P = \sum_{n=0}^{N} w(n) \cdot data^{n}
+       P = \\sum_{n=0}^{N} w(n) \\cdot data^{n}
 
     .. versionadded:: 0.10.0
 
