@@ -31,6 +31,8 @@ Standard plotting and mapping procedures
 # standard libraries
 import os.path as path
 import warnings
+import deprecation
+from deprecation import deprecated
 
 # site packages
 import numpy as np
@@ -50,6 +52,9 @@ from matplotlib.collections import LineCollection, PolyCollection
 # wradlib modules
 from . import georef as georef
 from . import util as util
+from .version import short_version
+
+deprecation.message_location = "top"
 
 
 class NorthPolarAxes(PolarAxes):
@@ -115,7 +120,7 @@ register_projection(NorthPolarAxes)
 
 
 def plot_ppi(data, r=None, az=None, autoext=True,
-             site=(0, 0), proj=None, elev=0.,
+             site=(0, 0, 0), proj=None, elev=0.,
              fig=None, ax=111, func='pcolormesh',
              cg=False, rf=1., refrac=False,
              **kwargs):
@@ -139,7 +144,7 @@ def plot_ppi(data, r=None, az=None, autoext=True,
     ``**kwargs` may be used to try to influence the
     :func:`matplotlib.pyplot.pcolormesh`, :func:`matplotlib.pyplot.contour`,
     :func:`matplotlib.pyplot.contourf` and
-    :meth:`wradlib.georef.polar2lonlatalt_n` routines under the hood.
+    :meth:`wradlib.georef.spherical_to_proj` routines under the hood.
 
     There is one major caveat concerning the values of `r` and `az`.
     Due to the way :func:`matplotlib.pyplot.pcolormesh` works, `r` should give
@@ -176,7 +181,7 @@ def plot_ppi(data, r=None, az=None, autoext=True,
         beam propagation will be taken into account. If False, simple
         trigonometry will be used to calculate beam propagation.
         Functionality for this will be provided by function
-        :meth:`wradlib.georef.arc_distance_n`. Therefore, if `refrac` is True,
+        :meth:`wradlib.georef.bin_distance`. Therefore, if `refrac` is True,
         `r` must be given in meters.
     site : tuple
         Tuple of coordinates of the radar site.
@@ -249,11 +254,13 @@ def plot_ppi(data, r=None, az=None, autoext=True,
 
     """
     # kwargs handling
-    kw_polar2lonlatalt_n = {}
+    kw_spherical = {}
     if 're' in kwargs:
-        kw_polar2lonlatalt_n['re'] = kwargs.pop('re')
+        re = kwargs.pop('re')
+        kw_spherical['re'] = re
     if 'ke' in kwargs:
-        kw_polar2lonlatalt_n['ke'] = kwargs.pop('ke')
+        ke = kwargs.pop('ke')
+        kw_spherical['ke'] = ke
     kwargs['zorder'] = kwargs.pop('zorder', 0)
 
     if (proj is not None) & cg:
@@ -298,7 +305,9 @@ def plot_ppi(data, r=None, az=None, autoext=True,
     if refrac & (proj is None):
         # with refraction correction, significant at higher elevations
         # calculate new range values
-        x = georef.arc_distance_n(x, elev)
+        re = kwargs.pop('re', 6370040.)
+        ke = kwargs.pop('ke', 4 / 3.)
+        x = georef.bin_distance(x, elev, site[2], re, ke=ke)
 
     # axes object is given
     if isinstance(ax, mpl.axes.Axes):
@@ -341,11 +350,14 @@ def plot_ppi(data, r=None, az=None, autoext=True,
             # if we produced a default, this one is still in 'kilometers'
             # therefore we need to get from km to m
             xx *= 1000
-        # latitude longitudes from the polar data still stored in xx and yy
-        lon, lat, alt = georef.polar2lonlatalt_n(xx, yy, elev, site,
-                                                 **kw_polar2lonlatalt_n)
+
         # projected to the final coordinate system
-        xx, yy = georef.reproject(lon, lat, projection_target=proj)
+        kw_spherical['proj'] = proj
+        coords = georef.spherical_to_proj(xx, yy, elev, site, **kw_spherical)
+
+        xx = coords[..., 0]
+        yy = coords[..., 1]
+
     else:
         if cg:
             yy = yy / rf
@@ -461,13 +473,10 @@ def plot_ppi_crosshair(site, ranges, angles=None,
         # these lines might not be straigt so we approximate them with 10
         # segments. Produce polar coordinates
         rr, az = np.meshgrid(np.linspace(0, ranges[-1], 10), angles)
-        # and reproject using polar2lonlatalt to convert from polar
-        # to geographic
-        nsewx, nsewy = georef.reproject(*georef.polar2lonlatalt_n(rr,
-                                                                  az,
-                                                                  elev,
-                                                                  site)[:2],
-                                        projection_target=proj)
+        # convert from spherical to projection
+        coords = georef.spherical_to_proj(rr, az, elev, site, proj=proj)
+        nsewx = coords[..., 0]
+        nsewy = coords[..., 1]
     else:
         # no projection
         psite = site
@@ -477,27 +486,23 @@ def plot_ppi_crosshair(site, ranges, angles=None,
                         psite[1] + rr * np.sin(np.radians(90 - az)))
 
     # mark the site, just in case nothing else would be drawn
-    ax.plot(*psite, marker='+', **linekw)
+    ax.plot(*psite[:2], marker='+', **linekw)
 
     # draw the lines
     for i in range(len(angles)):
         ax.add_line(mpl.lines.Line2D(nsewx[i, :], nsewy[i, :], **linekw))
 
     # draw the range circles
-    for r in ranges:
-        if proj:
-            # produce an approximation of the circle
-            x, y = georef.reproject(*georef.polar2lonlatalt_n(r,
-                                                              np.arange(360),
-                                                              elev,
-                                                              site)[:2],
-                                    projection_target=proj)
-            ax.add_patch(patches.Polygon(np.concatenate([x[:, None],
-                                                         y[:, None]],
-                                                        axis=1),
-                                         **circkw))
-        else:
-            # in the unprojected case, we may use 'true' circles.
+    if proj:
+        # produce an approximation of the circle
+        x, y = np.meshgrid(ranges, np.arange(360))
+        poly = georef.spherical_to_proj(x, y, elev, site, proj=proj)[..., :2]
+        poly = np.swapaxes(poly, 0, 1)
+        for p in poly:
+            ax.add_patch(patches.Polygon(p, **circkw))
+    else:
+        # in the unprojected case, we may use 'true' circles.
+        for r in ranges:
             ax.add_patch(patches.Circle(psite, r, **circkw))
 
     # there should be not much wrong, setting the axes aspect to equal
@@ -706,8 +711,12 @@ def plot_rhi(data, r=None, th=None, th_res=None, yoffset=0., autoext=True,
     if refrac:
         # observing air refractivity, so ground distances and beam height
         # must be calculated specially
-        xxx = georef.arc_distance_n(xx, yy) / rf
-        yyy = georef.beam_height_n(xx, yy) / rf
+        re = kwargs.pop('re', 6370040.)
+        ke = kwargs.pop('ke', 4/3.)
+        yyy = georef.bin_altitude(xx, yy, yoffset, re, ke=ke)
+        xxx = georef.site_distance(xx, yy, yyy, re, ke=ke)
+        xxx /= rf
+        yyy /= rf
         if cg:
             plax = caax
     else:
@@ -721,7 +730,7 @@ def plot_rhi(data, r=None, th=None, th_res=None, yoffset=0., autoext=True,
             xxx = xx * np.cos(np.radians(yy)) / rf
             yyy = xx * np.sin(np.radians(yy)) / rf
 
-    yyy += yoffset / rf
+        yyy += yoffset / rf
 
     # plot the stuff
     plotfunc = getattr(plax, func)
@@ -891,15 +900,14 @@ def create_cg(st, fig=None, subplot=111):
     return cgax, caax, paax
 
 
-@util.deprecated(plot_ppi)
+@deprecated(deprecated_in="0.10.0", removed_in="1.0.0",
+            current_version=short_version,
+            details="Use :func:`plot_ppi` with `cg=True` instead.")
 def plot_cg_ppi(data, r=None, az=None, rf=1.0, autoext=True,
                 refrac=True, elev=0., fig=None, subplot=111,
                 **kwargs):
     """Plots a Plan Position Indicator (PPI) on a curvelinear grid.
 
-    .. deprecated:: 0.11
-    Use :meth:`~wradlib.vis.plot_ppi`
-
     The implementation of this plot routine is in curvelinear grid axes and
     does all coordinate transforms beforehand. This allows zooming into the
     data as well as making it easier to plot additional data (like gauge
@@ -1043,14 +1051,13 @@ def plot_cg_ppi(data, r=None, az=None, rf=1.0, autoext=True,
     return cgax, caax, paax, pm
 
 
-@util.deprecated(plot_rhi)
+@deprecated(deprecated_in="0.10.0", removed_in="1.0.0",
+            current_version=short_version,
+            details="Use :func:`plot_rhi` with `cg=True` instead.")
 def plot_cg_rhi(data, r=None, th=None, th_res=None, autoext=True, refrac=True,
                 rf=1., fig=None, subplot=111, **kwargs):
     """Plots a Range Height Indicator (RHI) on a curvelinear grid.
 
-    .. deprecated:: 0.11
-    Use :meth:`~wradlib.vis.plot_rhi`
-
     The implementation of this plot routine is in a curvelinear grid axes and
     does all coordinate transforms beforehand.
 
@@ -1250,11 +1257,10 @@ def plot_scan_strategy(ranges, elevs, site, vert_res=500.,
     polc = util.meshgridN(ranges, az, elevs)
 
     # get mean height over radar
-    lon, lat, alt = georef.polar2lonlatalt_n(polc[0].ravel(), polc[1].ravel(),
-                                             polc[2].ravel(), site)
-    alt = alt.reshape(len(ranges), len(elevs))
+    coords, _ = georef.spherical_to_xyz(polc[0], polc[1], polc[2], site)
+    coords = np.squeeze(coords)
+    alt = coords[..., 2]
     r = polc[0].reshape(len(ranges), len(elevs))
-
     if ax is None:
         returnax = False
         fig = pl.figure()
